﻿Shader "Unlit/DepthExtrude"
{
    Properties
    {
        _MainTex ("Texture", 2D) = "white" {}
		_DepthMin ("Depth Min", Float) = 0
		_DepthMax ("Depth Max", Float) = 1
		_Gamma ("Gamma", Float) = 2.2
		_Threshold ("Threshold", Range(0,1)) = 0.1
    }
    SubShader
    {
        Tags { "RenderType"="Opaque" }
        LOD 100
		AlphaToMask On

        Pass
        {
            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag

            #include "UnityCG.cginc"

            struct appdata
            {
                float4 vertex : POSITION;
                float2 uv : TEXCOORD0;
            };

            struct v2f
            {
				float depth : TEXCOORD1;
                float2 uv : TEXCOORD0;
                float4 vertex : SV_POSITION;
            };

            sampler2D _MainTex;
			float _DepthMin;
			float _DepthMax;
			float _Gamma;
			float _Threshold;

			// http://lolengine.net/blog/2013/07/27/rgb-to-hsv-in-glsl
			float3 rgb2hsv(float3 c)
			{
				c = pow(saturate(c), 1/_Gamma);
				float4 K = float4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
				float4 p = lerp(float4(c.bg, K.wz), float4(c.gb, K.xy), step(c.b, c.g));
				float4 q = lerp(float4(p.xyw, c.r), float4(c.r, p.yzx), step(p.x, c.r));

				float d = q.x - min(q.w, q.y);
				float e = 1.0e-10;
				return float3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
			}

            v2f vert (appdata v)
            {
                v2f o;
                float4 depth = tex2Dlod(_MainTex, float4(v.uv * float2(0.5, 1) + float2(0.5, 0), 0, 0));
				float3 c = rgb2hsv(depth.rgb);
				if (c.b < 0.000001) {
					o.vertex = 0/sin(0);
					o.uv = 0;
					return o;
				}
				float d = c.r;
				v.vertex.z = 1;
				v.vertex.xyz *= lerp(_DepthMin, _DepthMax, d);

                o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				o.depth = d;
                return o;
            }

            fixed4 frag (v2f i) : SV_Target
            {
                float4 color = tex2D(_MainTex, i.uv * float2(0.5, 1));
                float4 depth = tex2D(_MainTex, i.uv * float2(0.5, 1) + float2(0.5, 0));
				float3 c = rgb2hsv(depth.rgb);
				float d = c.b;
				return float4(color.rgb, d * smoothstep(0.01,-0.01,abs(i.depth-c.r) - _Threshold));
            }
            ENDCG
        }
    }
}
